#
#
#  Strabo Example for ER-1 robots
#  Dave Evartt
#  25 Jan 03
#
# This is a simple demonstration of an ER-1 robot using Strabo Pathfinder 
# to move about in it's own little world.
# it uses the office.rm1 map and the patrol.txt to patrol the area.
# it does nothing interesting other than to patrol the area until it is shutdown.
# Feel free to load other maps and build other routefiles to suit your needs.
#
# This example shows how to make calls to Strabo and to translate directions
# into movement commands.
#
# It also shows how to read files.
# if while running, you press the override button on the server, this
# example will start using a different routefile. It is an example of sending commands back to the robot.
#
#


import urllib
import time
import string
import sys

# These imports are for the various functions of the ER-1
from ersp import task
from ersp.task import * 
from ersp.task import speech       # This is currently not implemented in this program
from ersp.task import navigation
from ersp.task import vision       # This is currently not implemented in this program
from ersp.task import util


# Sets the defaults for movement of the ER-1
task.setDefaultUnits(UNIT_DISTANCE, "in") 
task.setDefaultUnits(UNIT_ANGLE, "deg")
task.setDefaultUnits(UNIT_TIME, "sec")

false = 0
true = 1
orientation="N"
startpoint= "10,8"
endpoint=""
nestlocation=""
override=false
newCMD=""
speed = 10
distance = 12

def Strabo(cmd):
       global override
       global newCMD
       override=false
 #query = urllib.urlencode({'q':query})
       urlHandle = urllib.urlopen('http://localhost/' + cmd)
       returnData = urlHandle.read()
       execcmd= string.strip(returnData)
# handle command override
       if execcmd[0:4]=="CMD:":
          override=true
          returnData=execcmd[4:len(execcmd)]
          newCMD=returnData
       return returnData
    
def Turn(heading,oldheading):
       if oldheading =="N":
              if heading=="N":
                     return "forward"
              if heading=="E":
                     return "right"
              if heading=="S":
                     return "180"
              if heading=="W":
                     return "left"

       if oldheading =="E":
              if heading=="N":
                     return "left"
              if heading=="E":
                     return "forward"
              if heading=="S":
                     return "right"
              if heading=="W":
                     return "180"


       if oldheading =="S":
              if heading=="N":
                     return "180"
              if heading=="E":
                     return "left"
              if heading=="S":
                     return "forward"
              if heading=="W":
                     return "right"

       if oldheading =="W":
              if heading=="N":
                     return "right"
              if heading=="E":
                     return "180"
              if heading=="S":
                     return "left"
              if heading=="W":
                     return "forward"

       return ""


# move in whatever direction we need

def Movement(direction):
       global speed
       global distance
       
       if direction == "forward":
              navigation.MoveRelative(0, [distance, 0, speed],0)
       if direction =="right":
              navigation.TurnRelative(90)
              navigation.MoveRelative(0, [distance, 0, speed],0)
       if direction =="left":
              navigation.TurnRelative(-90)
              navigation.MoveRelative(0, [distance, 0, speed],0)
       if direction =="180":
              navigation.TurnRelative(180)
              navigation.MoveRelative(0, [distance, 0, speed],0)


 # move the robot to a waypoint
def GoToThe(waypoint):
    global startpoint
    global endpoint
    global orientation

    course=Strabo("getpath?"+startpoint +"," + waypoint)
    if override==true:
        print course
        return false        
    arry=course.split("\n")

    if string.strip(arry[2]) == "[0]": # it's already there
      speech.Speak( "Already there boss")
      return false

    if string.strip(arry[2]) == "[-1]": #can't get there
      speech.Speak("Can't get to " + waypoint + " there from here")      
      return false

    speech.Speak( "Headed for " + waypoint)
    startpoint= arry[2]
    startpoint = startpoint[1:len(startpoint)-1]
#    print startpoint
# do the course

    course = arry[3]
    course=course[1:len(course)-1]
 #   print course
    course = course.split(",")
    positionList = arry[5].split(",")
    counter=0
    for astep in course:
        time.sleep(.5)
        #speaker.Speak("bd ")
        counter=counter+1
        newdirect = Turn(astep,orientation)
        #Now move in the direction indicated
        print newdirect
        Movement(newdirect)        
        orientation=astep
        #print astep, positionList[counter].replace(":",",")  #move in this direction
        Strabo("setposition?"+positionList[counter].replace(":",","))
        if override==true:
            return false
    endpoint= arry[4]
    endpoint = endpoint[1:len(endpoint)-2]
    startpoint = endpoint # we need to keep track of our position
 #   print startpoint
    Strabo("report?All clear at "+waypoint)
    if override==true:
        return false
    return true

def main():
   global startpoint
   global nestlocation
   global newCMD
   routefile="patrol.txt"
   map = "office.rm1"
   print Strabo("loadmap?"+map)
       
   arry = string.split(Strabo("getposition"))
   startpoint= arry[1]
   
    
   print startpoint
#   time.sleep(4)
   battery = 0
   while 1:

 #Read a file
       in_file = open(routefile,"r")
       route = in_file.read()
       in_file.close()
       route = route.split("\n")
       print route
       
       for waypoint in route:
         battery = battery+1 # cheesy power input simulation

         
         if waypoint[0:1] == "n":
              nestlocation=waypoint[2:len(waypoint)]
              if battery>50: # if power is low as we go by the nest, go and recharge
                speech.Speak("Batteries are low! Need Power!")
                GoToThe(nestlocation)
                if override==true:
                    break
                battery=0
                #recharge
                
         if waypoint[0:1] == "w":
              GoToThe(waypoint[2:len(waypoint)])
              if override==true:
                    break
         if override==true:
            break
             
       if newCMD!="":
           speech.Speak("New Route ordered:"+newCMD)
           routefile=newCMD+".txt"
           newCMD=""
     
if (__name__ == "__main__"):
   main()