
##############################################################################
# Simple Python Demo Application that uses a replacement email               # 
# reader and sender which will work with any email server. Also shown        #
# are examples of tasks, parallel tasks, events, global variables, and       #
# running external applications                                              #          
#                                                                            #
# Provided by PMC and GBW at http://www.Robots4All.com                       #
#                                                                            #
# Known Problems:                                                            #
# 1.  SDK Motor control commands i.e. Move, Turn, MoveRelative etc. appear   #
#     to be unstable. The functions seem to ignore distance so we are using  #
#     sleep timers to stop the action. Also occationally a move command will #
#     be ignored so you have to give the opposite command to re-enable it.   #
#     Setting UNIT_DISTANCE to inches and UNIT_ANGLE to degrees causes       #
#     unreliable results as well.                                            #
# 2.  DetectMotion function seems to cause rapid loss of system resources    #
#     This function has been commented out until SDK patch is available.     #      
#                                                                            #
#     Hopefully these will be fixed in the next SDK release.                 #
##############################################################################

### Please fill in the constants below with your own information###
#smtpServer = 'mail.yourmailserver.com'
#RobotEmailAddress = 'RobotEmailAddress@RobotMailserver.com'
#MyReturnEmailAddress = 'You@YourMailServer.com'
#RobotPOP3Server = 'mail.RobotMailServer.com'
#RobotPOP3Login = 'RobotMailLogin'
#RobotPOP3Password = 'RobotMailPassword'
###################################################################

#look for commands in the email Subject Field
LookForEmailField = 'Subject' 

# How often (in Seconds) to check for email. Should be > 60
period = 100

# Grammar file to use. This one is located in the ER1 Python dir
# under sample_code
grammar_file = "voicecontrol-grammar.xml"

CR = '\r'
LF = '\n'
CRLF = CR+LF

## Units
UNIT_DISTANCE = 0
UNIT_ANGLE = 1

## Import what we need
import smtplib
import email 
import base64
import time
import os
import math
import string
import ersp.task.speech
import ersp._libevoctask
import ersp.task.navigation
from poplib import *
from ersp import task
from ersp.task import util
from MimeWriter import MimeWriter
try:
  from cStringIO import StringIO
except ImportError:
  from StringIO import StringIO

EmailFieldValue = {"News": "news", "Left" : "left", "Right" : "right", \
                   "Forward" : "forward", "Back" : "back", "Quit" : "Quit", \
                   "Picture" : "picture", "On" : "on", "Off" : "off", "Stop" : "stop"}

################################################################################   
# Name:        CheckMail                                                       #
#                                                                              #
# Arguments:   POP3Server name,Your POP3 Account Login,                        #
#              Your Password, Mail Field of interest, Text to match            #
#                                                                              #
# Description: Logs in to POP3 account, Checks for email                       #
#              that matches the value of the FROM, TO, SUBJECT,                #
#              or BODY fields of an email with a supplied value.               #
#              A match or finishing without a match throws an Event            #
#              None or the value of the match. A matched email                 #
#              gets marked for deletion and CheckMail logs off from the server.#
################################################################################

def CheckMail(context):
  (m_popserver,m_user,m_password,m_condition) = context.getArguments()
  fFlag = 0
  task = context.getTask()
  global MyReturnEmailAddress
  p = POP3(m_popserver) 
  p.user(m_user)
  p.pass_(m_password)
  numMessages = p.stat()[0]
  msgBuf = ""
  str = "Started Checking mail"
  print str
  ersp.task.speech.Speak(str,)
  for i in range(numMessages):
    for j in p.retr(i+1)[1]:
	msgBuf = msgBuf + j + CRLF
    msgStruct = email.message_from_string(msgBuf)
    #Uncomment these to see fields and values
    #print 'MSG:', i+1
    #print 'To:', msgStruct['To']
    #print 'From:', msgStruct['From']
    #print 'Subject:', msgStruct['Subject']
    #print 'Body:', msgStruct.get_payload()
    if(msgStruct[m_condition] != None):
      if (string.strip(msgStruct[m_condition]) in EmailFieldValue):
        if(string.strip(msgStruct['From'])):
          MyReturnEmailAddress = string.strip(msgStruct['From'])
        m_value = string.strip(msgStruct[m_condition])	
        fFlag = 1
        str = "found a match"
        print str
        ersp.task.speech.Speak(str,)
        b = p.dele(i+1)
        #print b
        str = "Deleted mail Message"
        print str
        ersp.task.speech.Speak(str,)
        p.noop()
        event = ersp.task.Event("CheckMail.Result")
        event ["Result"] = m_value
        manager = context.getTaskManager()
        manager.raiseEvent(event)
        i = numMessages + 1
        break
  p.noop
  p.quit()
  str = "Finished CheckMail, Logged off of POP3 server"
  print str
  ersp.task.speech.Speak(str,)

# Register Task
CMail = task.registerTask("CMail", CheckMail)


################################################################################   
# Name:        EventWatcher                                                    #
#                                                                              #
# Arguments:   None                                                            #
#                                                                              #
# Description: Looks for an Event of CheckMail.Result. If it is found          #
#              and is NOT equal to None, then it executes the command. If the  #
#              Command requires an external application this function will     #
#              spawn the external program or application with the supplied     #
#              arguments. After it executes the the command it sends email     #
#              confirmation. If the command is Picture it will change MIME     #
#              types, encode the image and attach it to the email reply.       #
################################################################################

def EventWatcher(context):
  global MyReturnEmailAddress
  while(not(context.terminationRequested())):
    task = context.getTask()
    e = task.waitForEvent("CheckMail.Result")
    if (e != None):
      str = "Got Event " + e['Result'] + " Trying to execute."
      print str
      fName = ""
      ersp.task.speech.Speak(str,)
      if(e['Result'] == "News"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\myinfo.exe', [' news'])
      elif(e['Result'] == "On"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\robox10.exe', [' A1 On'])
      elif(e['Result'] == "Off"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\robox10.exe', [' A1 Off'])
      elif(e['Result'] == "Left"):
        ersp.task.navigation.Turn(0.2)  
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['Result'] == "Right"):
        ersp.task.navigation.Turn(-0.2)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['Result'] == "Forward"):
        ersp.task.navigation.Move(5.0, 0.0)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['Result'] == "Back"):
        ersp.task.navigation.Move(-4.0, 0.0)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['Result'] == "Stop"):
        time.sleep(1)
        ersp.task.navigation.Stop(1)
      elif(e['Result'] == "Quit"):
        task.terminate()
        ersp.task.getTaskManager().shutdown()
        break
      elif(e['Result'] == "Picture"):
        image = ersp.task.resource.GetImage()
        if (image == None):
          print "No image."
        else:
          print "Took a picture, saving it to a file."
          ersp._libevoctask.ersp_image__write_jpg(image, "snapshot.jpg", 90)
          fName = "snapshot.jpg"
      elif(e['Result'] == "Quit"):
        task.terminate()
        ersp.task.getTaskManager().shutdown()
        break
      if(fName != ""):
        print "Creating Attachment..."  
        msg = StringIO()
        mw = MimeWriter(msg)
        mw.addheader('To', MyReturnEmailAddress)
        mw.addheader('From', RobotEmailAddress)
        mw.addheader('Subject', 'Picture message')
        mw.startmultipartbody('mixed')
        # start off with a text/plain part
        part = mw.nextpart()
        body = part.startbody('text/plain')
        body.write('This is a picture from your Robot :)')
        # now add an image part
        part = mw.nextpart()
        part.addheader('Content-Transfer-Encoding', 'base64')
        part.addheader("Content-Disposition", 'attachment; filename="%s"' % fName)
        body = part.startbody('image/jpeg')
        base64.encode(open(fName, 'rb'), body)
        # finish off
        mw.lastpart()
      else:
        msg = StringIO()
        mw = MimeWriter(msg)
        mw.addheader('To', MyReturnEmailAddress)
        mw.addheader('From', RobotEmailAddress)
        mw.addheader('Subject', 'I executed the command')
        body = mw.startbody('text/plain')
        body.write(e['Result'])

      str = "Done with Mail Event, Sending Mail Message"
      print str
      ersp.task.speech.Speak(str,)
      s = smtplib.SMTP()
      s.connect(smtpServer)       
      s.sendmail(RobotEmailAddress,MyReturnEmailAddress, msg.getvalue())
      s.quit()
      task = context.getTask()
      print "Watching for Event..."
    task.disableEvent("CheckMail.Result")

# Register Task
MailWatcher = task.registerTask("MailWatcher", EventWatcher)

################################################################################   
# Name:        EventWatcher2                                                   #
#                                                                              #
# Arguments:   None                                                            #
#                                                                              #
# Description: Looks for an Event of Evolution.SpeechRecognized. If it is      #
#              found and is NOT equal to None then it executes the command. If #
#              the Command requires an external application this function will #
#              spawn the external program or application with the supplied     #
#              arguments. After it executes the the command it says Done with  #
#              Speech Event.                                                   #
################################################################################

def EventWatcher2(context):
  global MyReturnEmailAddress
  fName = ""
  while(not(context.terminationRequested())):
    task = context.getTask()
    e = task.waitForEvent("Evolution.SpeechRecognized")
    if (e != None and e['confidence'] > 0.0) :
      print (e['text'], e['confidence'])
      if(e['text'] == "news"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\myinfo.exe', [' news'])
      elif(e['text'] == "on"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\robox10.exe', [' A1 On'])
      elif(e['text'] == "off"):
        os.spawnv(os.P_WAIT, 'C:\\program files\\er1\\robox10.exe', [' A1 Off'])
      elif(e['text'] == "left"):
        ersp.task.navigation.Turn(0.2)  
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['text'] == "right"):
        ersp.task.navigation.Turn(-0.2)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['text'] == "forward"):
        ersp.task.navigation.Move(5.0, 0.0)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['text'] == "back"):
        ersp.task.navigation.Move(-4.0, 0.0)
        time.sleep(3)
        ersp.task.navigation.Stop(1)
      elif(e['text'] == "stop"):
        time.sleep(1)
        ersp.task.navigation.Stop(1)
      elif(e['text'] == "quit"):
        task.terminate()
        ersp.task.getTaskManager().shutdown()
        break
      str = "Done with Speech Event"
      print str
      ersp.task.speech.Speak(str,)
      print "Watching for Speech Event..."
    #else:
      #time.sleep(1)
      #ersp.task.navigation.Stop()
    task.disableEvent("Evolution.SpeechRecognized")

# Register Task
SpeechWatcher = task.registerTask("SpeechWatcher", EventWatcher2)

################################################################################   
# Name:        DetectMotion                                                    #
#                                                                              #
# Arguments:   None                                                            #
#                                                                              #
# Description: Looks for an Event of Evolution.MotionDetected If it is found   #
#              and is NOT equal to None then it Creates an event of type       #
#              CheckMail.Result with a value of Picture. CheckMail takes the   #
#              Picture command as if it were an email command and executes it  #
#              taking a picture and attaching it to an email and sending it    #
################################################################################

def DetectMotion(context):
  task = context.getTask()
  while (not(context.terminationRequested())) :
    e = task.waitForEvent("Evolution.MotionDetected")
    if (e != None):
      print "Hey It Moved!"
      event = ersp.task.Event("CheckMail.Result")
      event ["Result"] = 'Picture'
      manager = context.getTaskManager()
      manager.raiseEvent(event)
    task.disableEvent("Evolution.MotionDetected")

HeyItMoved = task.registerTask("HeyItMoved", DetectMotion)

def main():
  #ersp.task.setDefaultUnits(UNIT_DISTANCE, 'in')
  #ersp.task.setDefaultUnits(UNIT_ANGLE, "deg")
  str = "Registering Tasks..."
  print str
  p = task.Parallel()
  p.addTask(MailWatcher)
  p.addTask(SpeechWatcher)

  # HeyitMoved and DetectMotion are commented out until the SDK MotionDetection is fixed.
  # Mainly DetectMotion causes a rapid loss of system resources.
  #p.addTask(HeyItMoved)
  #p.addTask(ersp.task.vision.DetectMotion, [20, 30])

  # Comment this out if you don't want to use Voice Recognition
  p.addTask(ersp.task.speech.RecognizeSpeech, [grammar_file,])

  # This calls the CheckMail task using period as the interval 
  p.addTask(ersp.task.util.DoPeriodically, [period, CMail,RobotPOP3Server, RobotPOP3Login,                                             RobotPOP3Password,LookForEmailField])

  p.waitForAllTasks()

##
## Run it
##

if (__name__ == "__main__"):
  main()
  ersp.task.getTaskManager().shutdown()
  


